package com.samelement.iot_notification.screen;

import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.VibrationEffect;
import android.os.Vibrator;
import android.view.View;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.preference.PreferenceManager;

import com.samelement.iot_notification.R;

import io.flutter.Log;

// alarm activity always vibrate and play a ringtone
public class AlarmActivity extends AppCompatActivity {

    private final String TAG = getClass().getSimpleName();
    private Vibrator vibrator;
    private MediaPlayer mediaPlayer;

    private TextView tvValue;
    private TextView tvDevice;
    private TextView tvTitle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        int themeId = preferences.getInt(IntentKey.KEY_THEME_ID, 0);

        final int notificationId = getIntent().getIntExtra(IntentKey.KEY_NOTIF_ID, -1);

        if (themeId < 10) {
            setContentView(R.layout.activity_alert);
        } else {
            setContentView(R.layout.activity_alert_dark);
        }


        if (themeId < 10) {
            setContentView(R.layout.activity_alarm);
        } else {
            setContentView(R.layout.activity_alarm_dark);
        }

        // initialize vibrator
        // Get instance of Vibrator from current Context
        vibrator = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);

        // initialize alarm sound
        Uri notification = RingtoneManager.getDefaultUri(RingtoneManager.TYPE_ALARM);
        mediaPlayer = MediaPlayer.create(getApplicationContext(), notification);
        mediaPlayer.setLooping(true);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
            setShowWhenLocked(true);
            setTurnScreenOn(true);
        } else {
            getWindow().addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED
                    | WindowManager.LayoutParams.FLAG_DISMISS_KEYGUARD
                    | WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON);
        }

        tvValue = findViewById(R.id.tvValue);
        tvDevice = findViewById(R.id.tvDevice);
        tvTitle = findViewById(R.id.tvTitle);

        Button btnClose = findViewById(R.id.btnClose);
        btnClose.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                dismiss(notificationId);
            }
        });

        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                dismiss(notificationId);
            }
        }, IntentKey.TIME_OUT);
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);

        // because single top
        setIntent(intent);
    }

    @Override
    protected void onResume() {
        super.onResume();
        // initialize variable from bundle values
        String value = getIntent().getStringExtra(IntentKey.EXTRA_VALUE);
        String title = getIntent().getStringExtra(IntentKey.EXTRA_TITLE);
        String device = getIntent().getStringExtra(IntentKey.EXTRA_DEVICE);

        tvValue.setText(value);
        tvDevice.setText(device);
        tvTitle.setText(title);
    }

    @Override
    protected void onStart() {
        super.onStart();
        ringAlarm();
        vibrateAlarm();
    }

    @Override
    protected void onDestroy() {
        try {
            vibrator.cancel();
            mediaPlayer.stop();
        } catch (Exception e) {
            Log.d(TAG, e.getMessage());
        }
        super.onDestroy();
    }

    // keep repeating
    private void vibrateAlarm() {
        try {
            if (vibrator != null && vibrator.hasVibrator()) {

                // The '0' here means to repeat indefinitely
                // '0' is actually the index at which the pattern keeps repeating from (the start)
                // To repeat the pattern from any other point, you could increase the index, e.g. '1'
                int repeatVibrate = 0;
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    vibrator.vibrate(
                            VibrationEffect.createWaveform(IntentKey.VIBRATE_PATTERN, repeatVibrate)
                    );
                } else {
                    vibrator.vibrate(IntentKey.VIBRATE_PATTERN, repeatVibrate);
                }
            }
        } catch (Exception e) {
            Log.d(TAG, e.getMessage());
        }
    }

    private void ringAlarm() {
        try {
            mediaPlayer.start();
        } catch (Exception e) {
            Log.d(TAG, e.getMessage());
        }
    }

    private void dismiss(int notificationId) {
        NotificationManager notificationManager = (NotificationManager)
                getSystemService(Context.NOTIFICATION_SERVICE);
        if (notificationManager != null && notificationId >= 0) {
            notificationManager.cancel(notificationId);
        }

        finish();
    }
}

